# Copyright (C) 2020 Ludvig Larsson aka Valmond ludviglarsson@protonmail.com
# This code is licensed under the terms of the GNU Public License, version 3 or later (GPL-3.0-or-later)

import sys
import os
import platform

import locator


"""""
Single file / folder / (later list.txt / regex): -> also generates link file!
10f [-i][-f][-r][-n] name foldername filename

-i = insert data (as opposed to download)
-f = folder
-u = update
-n = no error if linkfile already exists

# Get data, ONLY use the link file
10f name.10f

"""


def print_help(self):
    with open("translation.txt", "r") as f:
        data = f.read()
        data = data.replace("EXE", self.executable)
        print(data)


# Set up the way to call an executable depending on which OS we are on:
executable = './10f'
if platform.system() == 'Windows':
    executable = './10f.exe'


def print_specific_help(specific_help_type):
    if specific_help_type == '-i':
        print('Insert: use to insert new data:')
        print('example:')
        print(executable + ' -i dataname foldername filename')
        print(executable + ' -i -f dataname foldername [regex]')
        print('-x tries to upload the data right away (default 10 tries)')
        print(executable + ' -i -x dataname foldername filename [x-tries]')
        print(executable + ' -i -f -x dataname foldername regex [x-tries]')
    if specific_help_type == '-t':
        print('Download data from a link.')
        print('example:')
        print(executable + ' -t mylink.10f')
        print(executable + ' -t mylink.10f tmp/storage/personal')
        print('Download without any parameters does not need the -t')
        print(executable + ' mylink.10f')
    if specific_help_type == '-f':
        print('Folder: Inserts the folder as data.')
    if specific_help_type == '-u':
        print('Update / Refresh (use this if the data already exists).')
        print('example:')
        print(executable + ' -u dataname foldername filename')
        print(executable + ' -u -f dataname foldername  [regex]')
    if specific_help_type == '-n':
        print('Overwrites without error if the resulting linkfile exists.')
    if specific_help_type == '-c':
        print('Create a link')
        print('example:')
        print(executable + ' -c dataname')
    if specific_help_type == '-l':
        print('List information')
        print('example for the global database:')
        print('example for the global database:')
        print(executable + ' -l')
        print('example for a specific entry:')
        print(executable + ' -l dataname')
    if specific_help_type == '-s':
        print('Set wanted number of fingers')
        print('A bigger number equals your data will be (tried to be) shared more')
        print('which makes your computer likely to share more data too.')
        print('A smaller number, less.')
        print('example:')
        print(executable + ' -s dataname 10')
    if specific_help_type == '-d':
        print('Delete one of your datas or just their fingers')
        print('example:')
        print('Delete data and fingers:')
        print(executable + ' -d dataname')
        print('Delete only fingers:')
        print(executable + ' -d -f dataname')
    if specific_help_type == '-s':
        print('Sets the number of fingers we would like for this data')
        print('Note: each successful finger is a node that shares your data')
        print('For each finger you share their data as they share yours.')
        print('Default is 10')
        print('example:')
        print(executable + ' -s dataname 20')
    if specific_help_type == '-x':
        print('Tries to upload the data (e.g generate new fingers)')
        print('This is done by asking for sharing other nodes data in exchange.')
        print('example to try ten times (default value) to get new fingers:')
        print(executable + ' -x dataname')
        print('example to try a specific number of times to get new fingers:')
        print(executable + ' -x dataname 20')
        print('Note: the process is random but will not generate more fingers than specified, see -s and -l')
    if specific_help_type == '-p':
        print('Show or update configuration:')
        print('Show actual configuration:')
        print(executable + ' -s')
        print('Update configuration, set port value to 20000:')
        print(executable + ' -s port 20000')
    if specific_help_type == '-ip':
        print('Try to figure out your public IP address:')
        print('Example:')
        print(executable + ' -ip')
        print('Result:')
        print('123.45.67.89')
        print('Hint:')
        print('Set your public ip address like this:')
        print(executable + ' -p ip 123.45.67.89')
    if specific_help_type == '-test':
        print('Check our configuration is okay by connecting to the listener')
        print('over our ip:port configuration.')
        print('Example:')
        print(executable + ' -test')


    sys.exit(0)


def print_help():
    print('Tenfingers usage:')
    print('')
    print(executable + ' [-i][-f][-u][-n][-l] name [foldername] [filename][regex]')
    print(executable + ' -t linkfile [destfolder]')
    print('')
    print('-i [-f] [-x] = insert data (as opposed to download)')
    print('-u [-f] = update/refresh data')
    print('-f = use a folder, not a file')
    print('-c = create a link')
    print('-n = no error if linkfile already exists')
    print('-l [name] = list information')
    print('-s name number_of_fingers_wanted = set how many times you want to exchange shares for this data')
    print('-x name [N] = try to share this data N times (default 10)')
    print('-d [-f] name = delete your shared data')
    print('-p [name + value] show or update congfiguration')
    print('-ip try to figure out your public IP address')
    print('-h [-i,-t,-f,-r,-n,-c,-s,-x,-d] specific help')
    print('')
    print('Download data with a link, say link.10f')
    print('./10f link.10f')
    sys.exit(0)


if len(sys.argv) == 1:
    print_help()


cmd = {}
commands = {'-h', '-t', '-i', '-f', '-u', '-n', '-c', '-l', '-s', '-x', '-d', '-p', '-ip', '-test'}
specific_help = False
specific_help_type = False

# Specific help?  e.g 10f -h -X  <- where X is the command we want to know more about
if len(sys.argv) == 3 and sys.argv[1] == '-h':
    print('HELP')

# Redo the argv list, just removing any commands and keeping the values, e.g remove those starting with a minus
value = {}
values = 0
for i in range(1, len(sys.argv)):  # Start at 1 to skip the script name
    a = sys.argv[i]
    if a[:1] != '-':  # Not a Command
        value[values] = a
        values += 1

for c in sys.argv:
    if c[:1] == '-':
        if specific_help and not specific_help_type:
            specific_help_type = c

        if c == '-h':  # Specific help:
            specific_help = True
        if c in commands:
            cmd[c] = True
        else:
            print('Unknown command:', c)
            print_help()

if specific_help:
    if specific_help_type == False:
        print_help()
    else:
        print_specific_help(specific_help_type)
        sys.exit(-1)

noerror = '-n' in cmd

if '-i' in cmd:
    # executable + ' -i [-x] dataname foldername filename [x-tries]'
    # executable + ' -i -f [-x] dataname foldername [regex] [x-tries]'

    if values < 3:
        print('Error: Insufficient data.')
        print('')
        print_specific_help('-i')

    name = value[0]
    folder = value[1]
    extra_data = None if values <= 2 else value[2]  # filename for file, regex for folders

    from database_query import get_link_data_by_name
    if get_link_data_by_name(name):
        print('Error: [' + name + '] already exixts, use -u to update the data.')

    for s in value:
        print('V:', s)

    from insert_data import main as insert_data_main
    if '-f' in cmd: # variable length number of argv, regex is optional
        from insert_data import insert_folder

        print('Insert folder:')
        regex = '.*' if values <= 2 else extra_data

        insert_folder(name, folder, regex)
        #insert_data_main(['', '-f', name, folder, regex])  # To test

    else:
        from insert_data import insert_file
        print('Insert file:')
        insert_file(name, folder, extra_data)  # extra_data = the file name
        #insert_data_main(['', name, folder, extra_data])  # extra_data = the file name

    if '-x' in cmd:
        nof_tries = 10
        if '-f' in cmd:
            # Here we must use the regex, or we won't let user decide how many tries (just use the default ".*")
            if values == 4:
                nof_tries = int(value[3])
            else:
                print('when using -f and -x, you must give regex and tries. You can use the default regex ".*", example:')
                print(' -f -x dataname foldername .* 20')
        else:
            if values >= 4:
                nof_tries = int(value[3])
        print('TRIES:', nof_tries)

        from database_query import get_link_dataid_by_name
        from scheduler import check_data_finger
        data_id = get_link_dataid_by_name(name)
        check_data_finger(data_id, nof_tries, True)


    # make the link NO because it's stupid
    #if len(sys.argv) >= 3:
    #    from get_link_from_data import create_and_save_link
    #    create_and_save_link(sys.argv[2])

    sys.exit(0)

if '-c' in cmd:
    if len(sys.argv) == 3:
        name = sys.argv[2]
        print('Create link for', name)
        from get_link_from_data import create_and_save_link_database
        create_and_save_link_database(name)
    else:
        print_specific_help('-c')

    sys.exit(0)


if '-t' in cmd:
    if len(sys.argv) == 3 or len(sys.argv) == 4:
        destfolder = './'
        if len(sys.argv) == 4:
            destfolder = sys.argv[3]
        # def get_data(linkfile, dest_folder):
        from get_data_from_link import get_data
        get_data(sys.argv[2], destfolder)
        sys.exit(0)

if '-l' in cmd:
    # List stuff
    import database_query as dq
    from helpers import human_readable_file_size
    from configuration import get_configuration_value

    print('--------------------')
    if len(sys.argv) == 3:
        print('Show information for the data ['+sys.argv[2]+']')
        # Show the fingers for this entry:
        data_id = dq.get_link_dataid_by_name(sys.argv[2])
        data_version = dq.get_highest_version(data_id)
        print('Actual data version:', data_version)
        dq.flush_fingers(sys.argv[2])
    else:
        print('Show general information:')
        print('')
        # Flush generic data about nodes, data etc.
        our_data_size = dq.get_owned_files_size()
        shared_data_size = dq.get_shared_files_size()
        occupied_space_other = int((1000*shared_data_size) / get_configuration_value('max_storage_space_other_mb')*1024*1024) * 0.1

        print('# Our Owned data: ', human_readable_file_size(our_data_size))
        dq.flush_owned_data()

        print('')
        print('# Shared data data:')
        print('Shared data:', human_readable_file_size(shared_data_size) + ' of max ' + human_readable_file_size(get_configuration_value('max_storage_space_other_mb')*1024*1024) + ' [' + str(occupied_space_other) + '%]')
        dq.flush_shared_data()

        print('')
        print('# Known nodes:')
        dq.flush_known_nodes()

    print('--------------------')
    sys.exit(0)

if '-s' in cmd:  # 10f -s name nof_fingers
    if len(sys.argv) == 4:
        nof_fingers = int(sys.argv[3])
        if nof_fingers is not None and nof_fingers > 0:
            print('Update number of fingers is not yet implemented')
        else:
            print_specific_help('-f')
    sys.exit(0)

if '-x' in cmd:  # 10f -x name (tries=10)
    nof_tries = 10
    if len(sys.argv) == 4:
        nof_tries = int(sys.argv[3])

    if len(sys.argv) == 3 or len(sys.argv) == 4:
        name = sys.argv[2]
        from database_query import get_link_dataid_by_name
        from scheduler import check_data_finger
        data_id = get_link_dataid_by_name(name)
        check_data_finger(data_id, nof_tries, True)
    else:
        print_specific_help('-f')
    sys.exit(0)


if '-d' in cmd:  # 10f -d [-f] name
    name = None
    delete_fingers_only = False
    if len(sys.argv) == 3:
        name = sys.argv[2]
    if len(sys.argv) == 4 and sys.argv[2] == '-f':
        name = sys.argv[3]
        delete_fingers_only = True
    if name is not None:
        import database_query as dq
        if dq.delete_fingers(name):
            print('Fingers deleted for:', name)
        else:
            print('The data [' + str(name) + '] does not exist.')
        if not delete_fingers_only:
            if dq.delete_owned_data(name):
                owned_file_name = os.path.join(locator.module_path(), 'data/local/', (name + '.aes'))
                os.remove(owned_file_name)
                print('Deletion of owned data [' + str(name) + '] successful.')
            else:
                print('The owned data [' + str(name) + '] does not exist.')
        sys.exit(0)
    else:
        print('huh?')


if '-u' in cmd:
    from update_data import main as update_data_main
    if '-f' in cmd:
        print('Update folder:')
        if len(sys.argv) == 4:
            update_data_main(['', sys.argv[2], sys.argv[3]])  # To test
        if len(sys.argv) == 5:
            update_data_main(['', sys.argv[2], sys.argv[3], sys.argv[4]])  # To test
    else:
        print('Update file:')
        update_data_main(['', sys.argv[2], sys.argv[3], sys.argv[4]])  # To test

    sys.exit(0)


if '-p' in cmd:
    string_values = ['ip','unsecure_nick_name','swicharoo_policy', 'exclude_ip_list']
    integer_values = ['port', 'listen_read_bytes', 'max_simultaneous_connections', 'listen_timeout_seconds', 'server_send_package_size',
                      'max_storage_space_other_mb', 'swicharoo_minsize_accept_all', 'swicharoo_file_maxsize_kb']

    import database_connection as dc

    if len(sys.argv) == 4:
        name = sys.argv[2]
        value = sys.argv[3]

        print('Update configuration, set: ' + name + ' = ' + value)

        if name in string_values:
            dc.update("UPDATE configuration_string SET value = ? WHERE name = ?", (value, name))
            sys.exit(0)

        if name in integer_values:
            dc.update("UPDATE configuration_int SET value = ? WHERE name = ?", (value, name))
            sys.exit(0)

        print('The value does not exist in the database, use -s to display values.')
        sys.exit(0)

    else:
        # Flush the config tables:
        print('------------------------------')
        print('Tenfingers configuration:')
        padding = 29
        for name in string_values:
            value = dc.query_one("SELECT value FROM configuration_string WHERE name = ?", (name,))
            print(name + ' '*(padding - len(name)) + ': "' + str(value) + '"')
        print('')
        for name in integer_values:
            value = dc.query_one("SELECT value FROM configuration_int WHERE name = ?", (name,))
            print(name + ' '*(padding - len(name)) + ': ' + str(value))
        print('------------------------------')

    sys.exit(0)

if '-ip' in cmd:
    print('Trying to figure out the public ip address of this machine:', flush=True)
    from helpers import get_public_ip
    ip = get_public_ip()

    if ip != '127.0.0.1':
        print('Your IP address should be:', ip)
        print('Hint:')
        print('Set your public ip address like this:')
        print(executable + ' -p ip', ip)
    else:
        print('There were errors...')
    sys.exit(0)

if '-test' in cmd:
    print('Trying to connect to the public ip:port address of this machine.', flush=True)
    from socket_communication import SecureCommunication
    from configuration import get_configuration_value
    import database_query as dq
    from Crypto.PublicKey import RSA

    my_port = get_configuration_value('port')
    my_ip =  get_configuration_value('ip')
    _, my_rsa_pub_hex_key = dq.get_rsa_hex_keys()
    my_rsa_pub_key = RSA.importKey(bytes.fromhex(my_rsa_pub_hex_key))

    sc = SecureCommunication(my_ip, my_port, my_rsa_pub_key)
    if sc.is_active():
        print('Success!')
    else:
        print('Connection to ' + my_ip + ':' + str(my_port) + ' failed...')
        print('Your public ip:port must be routed to this PC,')
        print('and for the test to succeed, the listener must be running.')
    sys.exit(0)


#
# Download data from a link
if len(cmd) == 0:
    if len(sys.argv) >= 2:
        link_file = sys.argv[1]
        print('Load:', link_file, flush=True)

        from get_data_from_link import get_data_from_database_link

        destination_folder = './'
        if len(sys.argv) == 3:
            destination_folder = sys.argv

        get_data_from_database_link(link_file, destination_folder)

        sys.exit(0)





print_help()

































